"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var commandpost = require("commandpost");
var YAML = require("js-yaml");
var converter_1 = require("./converter");
var pkg = require("../package.json");
var root = commandpost
    .create("sw2dts [input_filename]")
    .version(pkg.version, "-v, --version")
    .option("-w, --with-query", "With GET query parameters.")
    .option("-s, --sort-props", "Sort type properties order.")
    .option("-o, --output <output_filename>", "Output to file.")
    .option("-n, --namespace <namespace>", "Use namespace.")
    .option("--stdin", "[Deprecated] Input from standard input.")
    .action(function (opts, args) {
    // TODO Delete '--stdin' option.
    if (opts.stdin) {
        console.warn("'--stdin' option is deprecated.");
    }
    if (args.input_filename && opts.stdin) {
        process.stderr.write("Invalid parameters!\n");
        process.exit(1);
        return;
    }
    var outputFilename = opts.output[0];
    var promise = null;
    if (args.input_filename) {
        promise = fromFile(args.input_filename);
    }
    else {
        promise = fromStdin();
    }
    promise.then(function (input) {
        var namespace = opts.namespace[0];
        var withQuery = opts.withQuery;
        var sortProps = opts.sortProps;
        return converter_1.convert(YAML.safeLoad(input), { namespace: namespace, withQuery: withQuery, sortProps: sortProps });
    }).then(function (model) {
        if (outputFilename) {
            fs.writeFileSync(outputFilename, model);
        }
        else {
            process.stdout.write(model);
        }
        process.exit(0);
    }).catch(errorHandler);
});
commandpost
    .exec(root, process.argv)
    .catch(errorHandler);
function fromStdin() {
    process.stdin.setEncoding("utf-8");
    return new Promise(function (resolve, reject) {
        var data = "";
        process.stdin
            .on("data", function (chunk) { return data += chunk; })
            .once("end", function () { return resolve(data); })
            .once("error", function (err) { return reject(err); });
        setTimeout(function () {
            if (data) {
                return;
            }
            reject();
        }, 1000);
    });
}
function fromFile(inputFileName) {
    return new Promise(function (resolve, reject) {
        var file = fs.readFileSync(inputFileName, { encoding: "utf-8" });
        resolve(file);
    });
}
function errorHandler(err) {
    if (err == null) {
        process.stdout.write(root.helpText());
        process.exit(0);
        return;
    }
    if (err instanceof Error) {
        console.error(err.stack);
    }
    else if (err) {
        console.error(err);
    }
    process.exit(1);
}
