"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dtsgenerator_1 = require("dtsgenerator");
function convert(data, options) {
    if (options === void 0) { options = {}; }
    if (!options.nameResolver) {
        options.nameResolver = PascalCaseNameResolver;
    }
    var namespace = (options.namespace || "");
    var jsonSchemas = [];
    for (var title in data.definitions) {
        var schema = data.definitions[title];
        fixRef(schema);
        schema.id = title;
        jsonSchemas.push(schema);
    }
    if (options.withQuery) {
        var _loop_1 = function (path) {
            var props = data.paths[path];
            if (!props.get || isEmpty(props.get.parameters)
                || props.get.parameters.some(function (v) { return v.in !== "query"; })) {
                return "continue";
            }
            var required = [];
            var properties = props.get.parameters.reduce(function (result, value) {
                result[value.name] = {
                    type: value.type,
                    items: value.items,
                    enum: value.enum,
                };
                if (value.required) {
                    required.push(value.name);
                }
                return result;
            }, {});
            var schema = {
                id: options.nameResolver(path, props, options),
                type: "object",
                properties: properties,
                required: required,
            };
            jsonSchemas.push(schema);
        };
        for (var path in (data.paths || [])) {
            _loop_1(path);
        }
    }
    if (options.sortProps) {
        jsonSchemas.filter(function (sd) { return !!sd.properties; }).forEach(function (sd) {
            sd.properties = Object.keys(sd.properties).sort().reduce(function (result, key) {
                result[key] = sd.properties[key];
                return result;
            }, {});
        });
    }
    return dtsgenerator_1.default(jsonSchemas).then(function (text) {
        if (!namespace) {
            return text;
        }
        text = text.split("\n").join("\n    ").trim();
        return "declare namespace " + namespace + " {\n    " + text + "\n}\n";
    });
}
exports.convert = convert;
function fixRef(obj) {
    for (var key in obj) {
        if (key === "$ref") {
            obj["$ref"] = obj["$ref"].split("/").pop();
        }
        else if (typeof obj[key] === "object") {
            fixRef(obj[key]);
        }
    }
}
function isEmpty(array) {
    return array ? !array.length : true;
}
/**
 * Default name resolver which resolve name as PascalCase from path string.
 */
function PascalCaseNameResolver(path, pathDefinition, options) {
    options = options || {};
    path = path.replace(/{[^}]*}/g, '').replace(/\/$/, '');
    path = path.replace(/(^|[\/_-])(\w)/g, function (substr) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return args[1].toUpperCase();
    });
    path = path.replace(/(\d+)(\w)/, function (substr) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return args[0] + args[1].toUpperCase();
    });
    return path;
}
exports.PascalCaseNameResolver = PascalCaseNameResolver;
